@@ -27,12 +27,13 @@ class FileDialog : public QDialog
     Q_OBJECT
 public:
     friend class FileChooserPortal;
+    friend class AppModel;
 
     explicit FileDialog(QDialog *parent = nullptr, Qt::WindowFlags flags = {});
     ~FileDialog() override;
     Q_DISABLE_COPY_MOVE(FileDialog)
 
-protected:
+public:
     KFileWidget *const m_fileWidget;
     KConfigGroup m_configGroup;
 };
