@@ -35,7 +35,6 @@
 #include <KWayland/Client/plasmawindowmodel.h>
 #include <KWayland/Client/registry.h>
 #include <KWayland/Client/surface.h>
-#include <KWayland/Client/xdgforeign.h>
 
 // system
 #include <fcntl.h>
@@ -137,14 +136,15 @@ WaylandIntegration::Stream WaylandIntegration::startStreamingRegion(const QRect
     return globalWaylandIntegration->startStreamingRegion(region, mode);
 }
 
-WaylandIntegration::Stream WaylandIntegration::startStreamingVirtual(const QString &name, const QSize &size, Screencasting::CursorMode mode)
+WaylandIntegration::Stream
+WaylandIntegration::startStreamingVirtual(const QString &name, const QString &description, const QSize &size, Screencasting::CursorMode mode)
 {
-    return globalWaylandIntegration->startStreamingVirtualOutput(name, size, mode);
+    return globalWaylandIntegration->startStreamingVirtualOutput(name, description, size, mode);
 }
 
-WaylandIntegration::Stream WaylandIntegration::startStreamingWindow(const QMap<int, QVariant> &win, Screencasting::CursorMode mode)
+WaylandIntegration::Stream WaylandIntegration::startStreamingWindow(KWayland::Client::PlasmaWindow *window, Screencasting::CursorMode mode)
 {
-    return globalWaylandIntegration->startStreamingWindow(win, mode);
+    return globalWaylandIntegration->startStreamingWindow(window, mode);
 }
 
 void WaylandIntegration::stopStreaming(uint node)
@@ -207,11 +207,6 @@ void WaylandIntegration::requestTouchUp(quint32 touchPoint)
     globalWaylandIntegration->requestTouchUp(touchPoint);
 }
 
-void WaylandIntegration::setParentWindow(QWindow *window, const QString &parentWindow)
-{
-    globalWaylandIntegration->setParentWindow(window, parentWindow);
-}
-
 KWayland::Client::PlasmaWindowManagement *WaylandIntegration::plasmaWindowManagement()
 {
     return globalWaylandIntegration->plasmaWindowManagement();
@@ -266,11 +261,11 @@ void WaylandIntegration::WaylandIntegrationPrivate::acquireStreamingInput(bool a
     }
 }
 
-WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreamingWindow(const QMap<int, QVariant> &win,
+WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreamingWindow(KWayland::Client::PlasmaWindow *window,
                                                                                                Screencasting::CursorMode cursorMode)
 {
-    auto uuid = win[KWayland::Client::PlasmaWindowModel::Uuid].toString();
-    return startStreaming(m_screencasting->createWindowStream(uuid, cursorMode), {{QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Window)}});
+    return startStreaming(m_screencasting->createWindowStream(window->uuid(), cursorMode),
+                          {{QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Window)}});
 }
 
 WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreamingOutput(QScreen *screen, Screencasting::CursorMode mode)
@@ -315,10 +310,12 @@ WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startS
                           });
 }
 
-WaylandIntegration::Stream
-WaylandIntegration::WaylandIntegrationPrivate::startStreamingVirtualOutput(const QString &name, const QSize &size, Screencasting::CursorMode mode)
+WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreamingVirtualOutput(const QString &name,
+                                                                                                      const QString &description,
+                                                                                                      const QSize &size,
+                                                                                                      Screencasting::CursorMode mode)
 {
-    return startStreaming(m_screencasting->createVirtualOutputStream(name, size, 1, mode),
+    return startStreaming(m_screencasting->createVirtualOutputStream(name, description, size, 1, mode),
                           {
                               {QLatin1String("size"), size},
                               {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Virtual)},
@@ -614,35 +611,6 @@ void WaylandIntegration::WaylandIntegrationPrivate::requestTouchUp(quint32 touch
     }
 }
 
-static const char *windowParentHandlePropertyName = "waylandintegration-parentHandle";
-void WaylandIntegration::WaylandIntegrationPrivate::setParentWindow(QWindow *window, const QString &parentHandle)
-{
-    if (!m_xdgImporter) {
-        return;
-    }
-
-    if (window->isVisible()) {
-        auto importedParent = m_xdgImporter->importTopLevel(parentHandle, window);
-        auto surface = KWayland::Client::Surface::fromWindow(window);
-        importedParent->setParentOf(surface);
-    }
-
-    window->setProperty(windowParentHandlePropertyName, parentHandle);
-    window->installEventFilter(this);
-}
-
-bool WaylandIntegration::WaylandIntegrationPrivate::eventFilter(QObject *watched, QEvent *event)
-{
-    const bool ret = WaylandIntegration::WaylandIntegration::eventFilter(watched, event);
-    QWindow *window = static_cast<QWindow *>(watched);
-    if (event->type() == QEvent::Expose && window->isExposed()) {
-        const QString parentHandle = window->property(windowParentHandlePropertyName).toString();
-        auto importedParent = m_xdgImporter->importTopLevel(parentHandle, window);
-        importedParent->setParentOf(KWayland::Client::Surface::fromWindow(window));
-    }
-    return ret;
-}
-
 void WaylandIntegration::WaylandIntegrationPrivate::authenticate()
 {
     if (!m_waylandAuthenticationRequested) {
@@ -675,9 +643,6 @@ void WaylandIntegration::WaylandIntegrationPrivate::initWayland()
         m_windowManagement = m_registry->createPlasmaWindowManagement(name, version, this);
         Q_EMIT waylandIntegration()->plasmaWindowManagementInitialized();
     });
-    connect(m_registry, &KWayland::Client::Registry::importerUnstableV2Announced, this, [this](quint32 name, quint32 version) {
-        m_xdgImporter = m_registry->createXdgImporter(name, std::min(version, quint32(1)), this);
-    });
     connect(m_registry, &KWayland::Client::Registry::interfacesAnnounced, this, [this] {
         m_registryInitialized = true;
         qCDebug(XdgDesktopPortalKdeWaylandIntegration) << "Registry initialized";
